import plugins
import socket
import struct
import json

class Plugin(plugins.BasePlugin):
    __name__ = 'minecraft'

    def run(self, config):
        '''
        Fetch the amount of active and max players
        add to /etc/nixstats.ini
        [minecraft]
        enabled=yes
        hosts=127.0.0.1:8000,127.0.0.2:8000...
        '''

        my_hosts = config.get('minecraft', 'hosts').split(',')
        result = {}
        for connection_string in my_hosts:
            try:
                # Connect
                s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                hostname = connection_string.split(':')[0]
                port = int(connection_string.split(':')[1])
                s.connect((hostname, port))

                # Send handshake + status request
                s.send(self.pack_data("\x00\x00" + self.pack_data(hostname.encode('utf8')) + self.pack_port(port) + "\x01"))
                s.send(self.pack_data("\x00"))

                # Read response
                self.unpack_varint(s)     # Packet length
                self.unpack_varint(s)     # Packet ID
                l = self.unpack_varint(s) # String length

                d = ""
                while len(d) < l:
                    d += s.recv(1024)

                # Close our socket
                s.close()
            except:
                pass

            results = {}

            try:
                players = json.loads(d.decode('utf8'))['players']
                results['online'] = int(players['online'])
                results['max'] = int(players['max'])
            except:
                results['online'] = 0
                results['max'] = 0
            result[str(connection_string.replace('.', '-'))] = results

        return result


    def unpack_varint(self, s):
        d = 0
        for i in range(5):
            b = ord(s.recv(1))
            d |= (b & 0x7F) << 7*i
            if not b & 0x80:
                break
        return d

    def pack_varint(self, d):
        o = ""
        while True:
            b = d & 0x7F
            d >>= 7
            o += struct.pack("B", b | (0x80 if d > 0 else 0))
            if d == 0:
                break
        return o

    def pack_data(self, d):
        return self.pack_varint(len(d)) + d

    def pack_port(self, i):
        return struct.pack('>H', i)

if __name__ == '__main__':
    Plugin().execute()



#!/usr/bin/env python
# -*- coding: utf-8 -*- 
import plugins
import subprocess


class Plugin(plugins.BasePlugin):
    __name__ = 'asterisk'

    def run(self, *unused):
        p = subprocess.Popen("sudo asterisk -rx 'core show calls' | grep 'active' | cut -f1 -d ' '", stdout=subprocess.PIPE, shell=True)
        p = p.communicate()[0].decode('utf-8').replace("\n", "")
        p = { "calls": p }
        return p

if __name__ == '__main__':
    Plugin().execute()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import plugins
import json


class Plugin(plugins.BasePlugin):
    __name__ = 'mdstat'

    def run(self, config):
        '''
        Monitor software raid status using mdadm
        pip install mdstat
        '''
        data = os.popen('sudo mdjson').read()
        results = {}

        try:
            data = json.loads(data)
        except Exception:
            return "Could not load mdstat data"

        for key, value in data['devices'].items():
            device = {}
            if(value['active'] is not True):
                device['active'] = 0
            else:
                device['active'] = 1
            if(value['read_only'] is not False):
                device['read_only'] = 1
            else:
                device['read_only'] = 0
            if(value['resync'] is not None):
                device['resync'] = 1
            else:
                device['resync'] = 0
            device['faulty'] = 0
            for disk, diskvalue in value['disks'].items():
                if diskvalue['faulty'] is not False:
                    device['faulty'] = device['faulty'] + 1
            results[key] = device
        return results

if __name__ == '__main__':
    Plugin().execute()



import plugins
import struct
import time
import memcache

class Plugin(plugins.BasePlugin):
    __name__ = 'memcached'

    def run(self, config):
        '''
        pip install python-memcached
        add to /etc/nixstats.ini
        [memcached]
        enabled=yes
        host=127.0.0.1
        port=11211
        '''
        prev_cache = self.get_agent_cache()  # Get absolute values from previous check
        try:
            socket = config.get('memcached', 'socket')
        except:
            socket = False
        try:
            # Connect
            if socket is False:
                mc = memcache.Client(['%s:%s' % (config.get('memcached', 'host'), config.get('memcached', 'port'))], debug=0)
            else:
                mc = memcache.Client(['unix:/%s' % socket], debug=0)
        except:
            return "Could not connect to memcached"

        non_delta = (
            'accepting_conns',
            'bytes',
            'uptime',
            'total_items',
            'total_connections',
            'time_in_listen_disabled_us',
            'threads',
            'rusage_user',
            'rusage_system',
            'reserved_fds',
            'pointer_size',
            'malloc_fails',
            'lrutail_reflocked',
            'listen_disabled_num',
            'limit_maxbytes',
            'hash_power_level',
            'hash_bytes',
            'curr_items',
            'curr_connections',
            'connection_structures',
            'conn_yields',
            'reclaimed'
        )
        delta_keys = (
            'auth_cmds',
            'auth_errors',
            'bytes_read',
            'bytes_written',
            'touch_misses',
            'touch_hits',
            'incr_misses',
            'incr_hits',
            'cas_misses',
            'cas_badval',
            'incr_hits',
            'get_misses',
            'get_hits',
            'expired_unfetched',
            'evictions',
            'evicted_unfetched',
            'delete_misses',
            'delete_hits',
            'decr_misses',
            'decr_hits',
            'crawler_reclaimed',
            'crawler_items_checked',
            'cmd_touch',
            'cmd_get',
            'cmd_set',
            'cmd_flush',
            'cmd_misses',
            'cmd_badval',
            'cmd_hits'
        )

        results = {}
        data = {}
        try:
            result = mc.get_stats()
            for key, key_value in enumerate(result[0][1]):
                value = result[0][1][key_value]
                key = key_value.lower().strip()
                if key in non_delta:
                    results[key] = float(value)
                elif key in delta_keys:
                    value = float(value)
                    results[key] = self.absolute_to_per_second(key, float(value), prev_cache)
                    data[key] = float(value)
                else:
                    pass
        except:
            return 'Could not fetch memcached stats'

        data['ts'] = time.time()
        self.set_agent_cache(data)
        return results


if __name__ == '__main__':
    Plugin().execute()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import plugins
import psutil

class Plugin(plugins.BasePlugin):
    __name__ = 'temp'

    def run(self, *unused):
        '''
        expirimental plugin used to collect temperature from system sensors
        plugin can be tested by running nixstatsagent test temp
        '''
        data = {}

        if sys.platform == "win32":
            try:
                import wmi
            except:
                return 'wmi module not installed.'
            try:
                w = wmi.WMI(namespace="root\OpenHardwareMonitor")
                temperature_infos = w.Sensor()
                for sensor in temperature_infos:
                    if sensor.SensorType==u'Temperature':
                        data[sensor.Parent.replace('/','-').strip('-')] = sensor.Value
                return data
            except:
                return 'Could not fetch temperature data from OpenHardwareMonitor.'
        if not hasattr(psutil, "sensors_temperatures"):
            return "platform not supported"

        try:
                temps = psutil.sensors_temperatures()
        except:
                return "can't read any temperature"

        for device, temp in temps.items():
            for value in temp:
                type = value[0]
                if value[0] == '':
                    type = device
                data[type] = value[1]
        return data


if __name__ == '__main__':
    Plugin().execute()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import urllib
import time
import plugins
import json


class Plugin(plugins.BasePlugin):
    __name__ = 'rabbitmq'

    def run(self, config):
        '''
        rabbitmq status page metrics
        '''
        def ascii_encode_dict(data):
            ascii_encode = lambda x: x.encode('ascii') if isinstance(x, unicode) else x
            return dict(map(ascii_encode, pair) for pair in data.items())

        results = dict()
        next_cache = dict()
        raw_response = urllib.urlopen(config.get('rabbitmq', 'status_page_url')).fp
        next_cache['ts'] = time.time()
        prev_cache = self.get_agent_cache()  # Get absolute values from previous check

        try:
            j = json.loads(raw_response.read(), object_hook=ascii_encode_dict)
        except Exception:
            return False

        results['published'] = self.absolute_to_per_second('published', j['message_stats']['publish'], prev_cache)
        results['ack'] = self.absolute_to_per_second('ack', j['message_stats']['ack'], prev_cache)
        results['deliver_get'] = self.absolute_to_per_second('ack', j['message_stats']['deliver_get'], prev_cache)
        results['redeliver'] = self.absolute_to_per_second('ack', j['message_stats']['redeliver'], prev_cache)
        results['deliver'] = self.absolute_to_per_second('ack', j['message_stats']['deliver'], prev_cache)

        results['messages'] = self.absolute_to_per_second('ack', j['queue_totals']['messages'], prev_cache)
        results['messages_ready'] = self.absolute_to_per_second('ack', j['queue_totals']['messages_ready'], prev_cache)
        results['messages_unacknowledged'] = self.absolute_to_per_second('ack', j['queue_totals']['messages_unacknowledged'], prev_cache)

        results['listeners'] = len(j['listeners'])

        results['consumers'] = j['object_totals']['consumers']
        results['queues'] = j['object_totals']['queues']
        results['exchanges'] = j['object_totals']['exchanges']
        results['connections'] = j['object_totals']['connections']
        results['channels'] = j['object_totals']['channels']

        next_cache = j
        next_cache['ts'] = time.time()
        self.set_agent_cache(next_cache)

        return results


if __name__ == '__main__':
    Plugin().execute()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import psutil
import plugins


class Plugin(plugins.BasePlugin):
    __name__ = 'network'

    def run(self, config):
        '''
        Network monitoring plugin.
        To only enable certain interfaces add below [network]:
        interfaces = eth1,eth3,...
        '''

        try:
            enabled_interfaces = config.get('network', 'interfaces').split(',')
        except:
            enabled_interfaces = False

        if enabled_interfaces is False:
            return psutil.net_io_counters(pernic=True)
        else:
	    returndata = {}
	    interfaces = psutil.net_io_counters(pernic=True)
            for interface in interfaces:
		if interface in enabled_interfaces:
		    returndata[interface] = interfaces[interface]
            return returndata
        return None


if __name__ == '__main__':
    Plugin().execute()



#!/usr/bin/env python
# -*- coding: utf-8 -*- 
import plugins
import subprocess

### You need to add `nixstats ALL=(ALL) NOPASSWD: /usr/sbin/kamctl` to /etc/sudoers in order for this to work
class Plugin(plugins.BasePlugin):
    __name__ = 'asterisk'

    def run(self, *unused):
        p = subprocess.Popen("sudo kamctl ul show | grep AOR | wc -l", stdout=subprocess.PIPE, shell=True)
        p = p.communicate()[0].decode('utf-8').replace("\n", "")
        p = { "devices_online": p }
        return p

if __name__ == '__main__':
    Plugin().execute()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import plugins

class Plugin(plugins.BasePlugin):
    __name__ = 'exim'

    def run(self, config):
        '''
        exim mail queue monitoring, needs sudo access!
        Instructions at:
        https://help.nixstats.com/en/article/monitoring-exim-mail-queue-size-1vcukxa/
        '''
        data = {}
        data['queue_size'] = int(os.popen('sudo exim -bpc').read())
        return data

if __name__ == '__main__':
    Plugin().execute()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#

import os
import signal
import subprocess
import sys
import psutil
import plugins

def diskstats_parse(dev=None):
    file_path = '/proc/diskstats'
    result = {}

    if not os.path.isfile("/proc/diskstats"):
        return False

    # ref: http://lxr.osuosl.org/source/Documentation/iostats.txt
    columns_disk = ['m', 'mm', 'dev', 'reads', 'rd_mrg', 'rd_sectors',
                    'ms_reading', 'writes', 'wr_mrg', 'wr_sectors',
                    'ms_writing', 'cur_ios', 'ms_doing_io', 'ms_weighted']

    columns_partition = ['m', 'mm', 'dev', 'reads', 'rd_sectors', 'writes', 'wr_sectors']

    lines = open(file_path, 'r').readlines()
    for line in lines:
        if line == '':
            continue
        split = line.split()
        if len(split) == len(columns_disk):
            columns = columns_disk
        elif len(split) == len(columns_partition):
            columns = columns_partition
        else:
            # No match
            continue

        data = dict(zip(columns, split))

        if data['dev'][:3] == 'nvm' and data['dev'][-2:-1] == 'n':
            pass
        elif data['dev'][-1:].isdigit() is True:
            continue

        if "loop" in data['dev'] or "ram" in data['dev']:
            continue

        if dev is not None and dev != data['dev']:
            continue
        for key in data:
            if key != 'dev':
                data[key] = int(data[key])
        result[data['dev']] = data

    return result


class Plugin(plugins.BasePlugin):
    __name__ = 'iostat'

    def run(self, *unused):
        results = diskstats_parse()
        if not results  or results is False:
            results = {}
            try:
                diskdata = psutil.disk_io_counters(perdisk=True)
                for device, values in diskdata.items():
                    device_stats = {}
                    for key_value in values._fields:
                        device_stats[key_value] = getattr(values, key_value)
                    results[device] = device_stats
            except Exception as e:
                results = e.message
        return results


if __name__ == '__main__':
    Plugin().execute()



import re, sys, os
import libvirt
import libxml2
import time
import plugins

class Plugin(plugins.BasePlugin):
    __name__ = 'vms'

    def run(self, config):
        '''
        Using the libvirt API to fetch statistics from guests
        running KVM, QEMU, Xen, Virtuozzo, VMWare ESX, LXC,
        BHyve and more
        '''
        results = {}
        last_value = {}
        prev_cache = self.get_agent_cache()  # Get absolute values from previous check
        uri = os.getenv("uri", "qemu:///system")
        values = self.fetch_values(uri)

        deltas = {}
        for key, value in values.items():
            deltas[key] = {}
            for subkey, subvalue in value.items():
                deltas[key][subkey] = self.absolute_to_per_second('%s_%s' % (key, subkey), float(subvalue), prev_cache)
                last_value['%s_%s' % (key, subkey)] = float(value[subkey])
        last_value['ts'] = time.time()
        self.set_agent_cache(last_value)
        return results

    def canon(self, name):
        return re.sub(r"[^a-zA-Z0-9_]", "_", name)

    def get_ifaces(self, dom):
        xml = dom.XMLDesc(0)
        doc = None
        try:
            doc = libxml2.parseDoc(xml)
        except:
            return []
        ctx = doc.xpathNewContext()
        ifaces = []
        try:
            ret = ctx.xpathEval("/domain/devices/interface")
            for node in ret:
                devdst = None
                for child in node.children:
                    if child.name == "target":
                        devdst = child.prop("dev")
                if devdst == None:
                    continue
                ifaces.append(devdst)
        finally:
            if ctx != None:
                ctx.xpathFreeContext()
            if doc != None:
                doc.freeDoc()
        return ifaces

    def get_memtune(self, dom):
        memtune = { 'min_guarantee': 0, 'soft_limit': 0, 'hard_limit': 0 }
        xml = dom.XMLDesc(0)

        try:
            doc = libxml2.parseDoc(xml)
        except:
            return []

        ctx = doc.xpathNewContext()
        try:
            for key in memtune:
                ret = ctx.xpathEval("/domain/memtune/%s" % key)
                try:
                    for child in ret[0].children:
                        memtune[key] = int(child.content)
                        break
                except IndexError:
                        # key not found in xml
                        pass
        finally:
            if ctx != None:
                ctx.xpathFreeContext()
            if doc != None:
                doc.freeDoc()
        return memtune

    def fetch_values(self, uri):
        conn = libvirt.openReadOnly(uri)
        ids = conn.listDomainsID()
        results = {}
        processors = float(conn.getInfo()[2])
        data = {}
        for id in ids:
            data['net_rx'] = 0
            data['net_tx'] = 0
            try:
                dom = conn.lookupByID(id)
                name = dom.name()
            except libvirt.libvirtError, err:
                print >>sys.stderr, "Id: %s: %s" % (id, err)
                continue
            if name == "Domain-0":
                continue
            ifaces = self.get_ifaces(dom)
            for iface in ifaces:
                try:
                    stats = dom.interfaceStats(iface)
                    data['net_rx'] += stats[0]
                    data['net_tx'] += stats[4]
                except:
                    print >>sys.stderr, "Cannot get ifstats for '%s' on '%s'" % (iface, name)

            cputime = float(dom.info()[4])
            cputime_percentage = 1.0e-7 * cputime / processors
            data['cpu'] = cputime_percentage

            maxmem, mem = dom.info()[1:3]
            mem *= 1024
            maxmem *= 1024
            data['mem'] = mem
            memtune = self.get_memtune(dom)
            data['min_guarantee'] = memtune['min_guarantee'] * 1024
            data['hard_limit'] = memtune['hard_limit'] * 1024
            data['soft_limit'] = memtune['soft_limit'] * 1024

            data['disk_rd'] = 0
            data['disk_wr'] = 0
            data['disk_wr_req'] = 0
            data['disk_rd_req'] = 0
            try:
                dom = conn.lookupByID(id)
                name = dom.name()
            except libvirt.libvirtError, err:
                print >>sys.stderr, "Id: %s: %s" % (id, err)
                continue
            if name == "Domain-0":
                continue
            disks = self.get_disks(dom)
            for disk in disks:
                try:
                    rd_req, rd_bytes, wr_req, wr_bytes, errs = dom.blockStats(disk)
                    data['disk_rd'] += rd_bytes
                    data['disk_wr'] += wr_bytes
                    data['disk_rd_req'] += rd_req
                    data['disk_wr_req'] += wr_req
                except TypeError:
                    print >>sys.stderr, "Cannot get blockstats for '%s' on '%s'" % (disk, name)

            results[self.canon(name)] = data
        return results

    def get_disks(self, dom):
        xml = dom.XMLDesc(0)
        doc = None
        try:
            doc = libxml2.parseDoc(xml)
        except:
            return []
        ctx = doc.xpathNewContext()
        disks = []
        try:
            ret = ctx.xpathEval("/domain/devices/disk")
            for node in ret:
                devdst = None
                for child in node.children:
                    if child.name == "target":
                        devdst = child.prop("dev")
                if devdst == None:
                    continue
                disks.append(devdst)
        finally:
            if ctx != None:
                ctx.xpathFreeContext()
            if doc != None:
                doc.freeDoc()
        return disks


if __name__ == '__main__':
    Plugin().execute()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import plugins
import sys

class Plugin(plugins.BasePlugin):
    __name__ = 'loadavg'

    def run(self, *unused):
        if sys.platform == 'win32':
            return None
        else:
            return os.getloadavg()


if __name__ == '__main__':
    Plugin().execute()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import psutil
import plugins
import json


class Plugin(plugins.BasePlugin):
    __name__ = 'diskusage'


    def run(self, config):
        disk = {}
        disk['df-psutil'] = []

        for part in psutil.disk_partitions(False):
            if os.name == 'nt':
                if 'cdrom' in part.opts or part.fstype == '':
                    # skip cd-rom drives with no disk in it; they may raise
                    # ENOENT, pop-up a Windows GUI error for a non-ready
                    # partition or just hang.
                    continue
            try:
                usage = psutil.disk_usage(part.mountpoint)
                diskdata = {}
                diskdata['info'] = part
                for key in usage._fields:
                    diskdata[key] = getattr(usage, key)
                disk['df-psutil'].append(diskdata)
            except:
                pass

        try:
            force_df = config.get('diskusage', 'force_df')
        except:
            force_df = 'no'

        if len(disk['df-psutil']) == 0 or force_df == 'yes':
            try:
                disk['df-psutil'] = []
                df_output_lines = [s.split() for s in os.popen("df -Pl").read().splitlines()]
                del df_output_lines[0]
                for row in df_output_lines:
                    if row[0] == 'tmpfs':
                        continue
                    disk['df-psutil'].append({'info': [row[0], row[5],'',''], 'total': int(row[1])*1024, 'used': int(row[2])*1024, 'free': int(row[3])*1024, 'percent': row[4][:-1]})
            except:
                pass
        try:
            lvm_stats = config.get('diskusage', 'lvm')
        except:
            lvm_stats = 'no'

        # For LVM volume group monitoring, requires sudo access to vgs
        # add vgs to /etc/sudoers
        # nixstats ALL=(ALL) NOPASSWD: /usr/sbin/vgs
        # set lvm = yes after enabled = yes in /etc/nixstats.ini
        if lvm_stats == 'yes':
            try:
                lines = [s.split(', ') for s in os.popen("vgs --all --units b --noheadings --separator ', '").read().splitlines()]
                for row in lines:
                    v = {}
                    v['vg_name'] = row[0]
                    v['vg_size'] = int(row[5][:-1])
                    v['vg_free'] = int(row[6][:-1])
                    v['vg_used'] = int(v['vg_size']-v['vg_free'])
                    v['vg_percentage'] = (v['vg_used']/float(v['vg_size']))*100
                    disk['df-psutil'].append({'info': [v['vg_name'], v['vg_name'], 'lvm', False], 'total': v['vg_size'], 'used': v['vg_used'], 'free': v['vg_free'], 'percent': v['vg_percentage']})
            except Exception as e:
                return e.message


        return disk


if __name__ == '__main__':
    Plugin().execute()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import plugins
import redis

### Uncomment/Comment the Attribute Names to be monitored
METRICS = {
# Server section
    #"redis_version": "redis version",
    #"redis_git_sha1": "redis git sha1",
    #"redis_git_dirty": "redis git dirty", 
    #"redis_build_id": "redis build id", 
    #"redis_mode": "redis mode", 
    #"os": "os",    
    #"arch_bits": "arch bits", 
    #"multiplexing_api": "multiplexing api", 
    #"gcc_version": "gcc version",
    #"process_id": "process id",
    #"run_id": "run id", 
    #"tcp_port": "tcp port",
    "uptime_in_seconds": "uptime", 
    #"uptime_in_days": "uptime in days", 
    #"hz": "hz", 
    #"lru_clock": "lru clock", 
    #"executable": "redis path",
    #"config_file": "config file",

# Clients section
    "connected_clients": "connected clients",
    #"client_longest_output_list": "client longest output list", 
    #"client_biggest_input_buf": "client biggest input buf", 
    #"blocked_clients": "blocked clients", 

# Memory section
    "used_memory": "used memory", 
    #"used_memory_human": "used memory human", 
    #"used_memory_rss": "used memory rss",
    #"used_memory_rss_human": "used memory rss human",
    #"used_memory_peak": "used memory peak", 
    "used_memory_peak_human": "used memory peak human", 
    #"total_system_memory": "total system memory",
    #"total_system_memory_human": "total system memory human",
    #"used_memory_lua": "used memory lua",
    #"used_memory_lua_human": "used memory lua human",
    "maxmemory": "maxmemory",
    #"maxmemory_human": "maxmemory human",
    "maxmemory_policy": "maxmemory policy",
    #"mem_fragmentation_ratio": "mem fragmentation ratio",
    #"mem_allocator": "mem allocator",

# Persistence section     
    #"loading": "loading",
    #"rdb_changes_since_last_save": "rdb changes since last save",
    #"rdb_bgsave_in_progress": "rdb bgsave in progress",
    #"rdb_last_save_time": "rdb last save time",
    #"rdb_last_bgsave_status": "rdb last bgsave status",
    #"rdb_last_bgsave_time_sec": "rdb last bgsave time sec",
    #"rdb_current_bgsave_time_sec": "rdb current bgsave time sec",
    #"aof_enabled": "aof enabled",
    #"aof_rewrite_in_progress": "aof rewrite in progress",
    #"aof_rewrite_scheduled": "aof rewrite scheduled",    
    #"aof_last_rewrite_time_sec": "aof last rewrite time",
    #"aof_current_rewrite_time_sec": "aof current rewrite time",
    #"aof_last_bgrewrite_status": "aof last bgrewrite status",
    #"aof_last_write_status": "aof last write status",
    #"aof_current_size": "aof current size",
    #"aof_base_size": "aof base size",
    #"aof_pending_rewrite": "aof pending rewrite",
    #"aof_buffer_length": "aof buffer length",
    #"aof_rewrite_buffer_length": "aof rewrite buffer length",
    #"aof_pending_bio_fsync": "aof pending bio fsync",
    #"aof_delayed_fsync": "aof delayed fsync",

# Stats section
    #"total_connections_received": "total connections received",
    "total_commands_processed": "total commands processed",
    #"instantaneous_ops_per_sec": "instantaneous ops per sec",
    "total_net_input_bytes": "total net input bytes",
    "total_net_output_bytes": "total net output bytes",
    #"instantaneous_input_kbps": "instantaneous input kbps",
    #"instantaneous_output_kbps": "instantaneous output kbps",
    #"rejected_connections": "rejected connections",
    #"sync_full": "sync full",
    #"sync_partial_ok": "sync partial ok",
    #"sync_partial_err": "sync partial err",
    "expired_keys": "expired keys",
    "evicted_keys": "evicted keys",
    "keyspace_hits": "keyspace hits",
    "keyspace_misses": "keyspace misses",
    #"pubsub_channels": "pubsub channels",
    #"pubsub_patterns": "pubsub patterns",
    #"latest_fork_usec": "latest fork usec",
    #"migrate_cached_sockets": "migrate cached sockets",

# Replication section
    #"role": "role",
    #"connected_slaves": "connected slaves",
    #"master_repl_offset": "master repl offset",
    #"repl_backlog_active": "repl backlog active",
    #"repl_backlog_size": "repl backlog size",
    #"repl_backlog_first_byte_offset": "repl backlog first byte offset",
    #"repl_backlog_histlen": "repl backlog histlen",

# CPU section
    #"used_cpu_sys": "used cpu sys",
    #"used_cpu_user": "used cpu user",
    #"used_cpu_sys_children": "used cpu sys children",
    #"used_cpu_user_children": "used cpu user children",    

# Cluster section
    "cluster_enabled": "cluster enabled"
}

class Plugin(plugins.BasePlugin):
    __name__ = 'redis_stat'

    def run(self, config):
        data = {}
        stats = None
        try:
            redis_host = (config.get(__name__, 'host'))
        except:
            redis_host = '127.0.0.1'
        try:
            redis_port = (config.get(__name__, 'port'))
        except:
            redis_port = '6379'
        try:
            redis_db = (config.get(__name__, 'db'))
        except:
            redis_db = '0'
        try:
            redis_password = (config.get(__name__, 'password'))
        except:
            redis_password = ''

        try:
            redis_connection = redis.StrictRedis(host=redis_host,port=redis_port,db=redis_db,password=redis_password)
            stats = redis_connection.info()
        except Exception as e:
            data['status']=0
            data['msg']='Connection Error'
            if not stats:
                return data

        for name, value in stats.iteritems():
            if name in METRICS.keys() :
                data[METRICS[name]] = value
        return data

if __name__ == '__main__':
    Plugin().execute()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import plugins
import time


class Plugin(plugins.BasePlugin):
    __name__ = 'docker'

    def run(self, config):
        '''
        Docker monitoring, needs sudo access!
        Instructions at:
        https://help.nixstats.com/en/article/monitoring-docker-9st778/
        '''
        containers = {}
        last_value = {}
        prev_cache = self.get_agent_cache()  # Get absolute values from previous check
        try:
            lines = [s.split(' / ') for s in os.popen('sudo docker stats --no-stream --no-trunc --format "{{.CPUPerc}} / {{.Name}} / {{.ID}} / {{.MemUsage}} / {{.NetIO}} / {{.BlockIO}} / {{.MemPerc}}"').read().splitlines()]
            for row in lines:
                container = {}
                container['cpu'] = row[0].strip('%')
                name = row[1]
                container_id = row[2]
                container['mem_usage_bytes'] = self.computerReadable(row[3])
                container['mem_total_bytes'] = self.computerReadable(row[4])
                container['net_in_bytes'] = self.absolute_to_per_second('%s_%s' % (name, 'net_in_bytes'), self.computerReadable(row[5]), prev_cache)
                container['net_out_bytes'] = self.absolute_to_per_second('%s_%s' % (name, 'net_out_bytes'), self.computerReadable(row[6]), prev_cache)
                container['disk_in_bytes'] = self.absolute_to_per_second('%s_%s' % (name, 'disk_in_bytes'), self.computerReadable(row[7]), prev_cache)
                container['disk_out_bytes'] = self.absolute_to_per_second('%s_%s' % (name, 'disk_out_bytes'), self.computerReadable(row[8]), prev_cache)
                container['mem_pct'] = row[9].strip('%')

                last_value['%s_%s' % (name, 'mem_usage_bytes')] = self.computerReadable(row[3])
                last_value['%s_%s' % (name, 'net_in_bytes')] = self.computerReadable(row[5])
                last_value['%s_%s' % (name, 'net_out_bytes')] = self.computerReadable(row[6])
                last_value['%s_%s' % (name, 'disk_in_bytes')] = self.computerReadable(row[7])
                last_value['%s_%s' % (name, 'disk_out_bytes')] = self.computerReadable(row[8])
                containers[name] = container
        except Exception as e:
            return e.message
        containers['containers'] = len(containers)
        last_value['ts'] = time.time()
        self.set_agent_cache(last_value)

        return containers

    def computerReadable(self, value):
        if value[-3:] == 'KiB':
            return float(value[:-3])*1024
        elif value[-3:] == 'MiB':
            return float(value[:-3])*1024*1024
        elif value[-3:] == 'GiB':
            return float(value[:-3])*1024*1024*1024
        elif value[-3:] == 'TiB':
            return float(value[:-3])*1024*1024*1024*1024
        elif value[-3:] == 'PiB':
            return float(value[:-3])*1024*1024*1024*1024*1024
        elif value[-2:] == 'kB':
            return float(value[:-2])*1024
        elif value[-2:] == 'MB':
            return float(value[:-2])*1024*1024
        elif value[-2:] == 'GB':
            return float(value[:-2])*1024*1024*1024
        elif value[-2:] == 'TB':
            return float(value[:-2])*1024*1024*1024*1024
        elif value[-2:] == 'PB':
            return float(value[:-2])*1024*1024*1024*1024*1024
        elif value[-1:] == 'B':
            return float(value[:-1])

if __name__ == '__main__':
    Plugin().execute()
